subroutine limiter_f(wk,wL,wR,ix,jx,type)
!-----------------------------------------------------------------------
!     Slope limiters
!-----------------------------------------------------------------------
!     2010/01/22  S. Zenitani  2nd order limiters (minmod, MC)
!     2010/05/14  S. Zenitani  added van Leer limiter
!-----------------------------------------------------------------------
  implicit none
  integer :: ix, jx
  real(8) :: wk(ix,jx)             ! work array (input)
  real(8) :: wL(ix,jx), wR(ix,jx)  ! interpolated values (output)
  integer :: type            ! 0 (flat), 1 (minmod), 2 (MC), 3 (van Leer)
  integer :: i, j
  real(8) :: gA, gB, gC      ! average, left, and right gradients
  real(8) :: grad

! 1st order
  if( type .eq. 0 ) then
     do j=1,jx
        do i=1,ix-1
           wL(i,j) = wk(i,j)
           wR(i,j) = wk(i+1,j)
        enddo
     enddo

! 2nd order
! minmod limiter
  elseif( type .eq. 1 ) then

     do i=1,1
        wL(i,:)   = wk(i,:)
     enddo
     do j=1,jx
        do i=2,ix-1
           gA =     ( wk(i,j)  -wk(i-1,j))
           gB =     ( wk(i+1,j)-wk(i,j)  )
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              if( gA .gt. 0 ) then
                 grad = min(gA,gB)
              else
                 grad = max(gA,gB)
              endif
           endif
           wR(i-1,j) = wk(i,j) - 0.5d0 * grad
           wL(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do i=ix,ix
        wR(i-1,:) = wk(i,:)
     enddo
         
! 2nd order
! Monotonized central-difference (MC) limitter
  elseif( type .eq. 2 ) then
     
     do i=1,1
        wL(i,:)   = wk(i,:)
     enddo
     do j=1,jx
        do i=2,ix-1
           gA =       ( wk(i,j)  -wk(i-1,j))
           gB =       ( wk(i+1,j)-wk(i,j)  )
           gC = 0.5d0*( wk(i+1,j)-wk(i-1,j))
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              if( gA .gt. 0 ) then
                 grad = min(2*gA,2*gB,gC)
              else
                 grad = max(2*gA,2*gB,gC)
              endif
           endif
           wR(i-1,j) = wk(i,j) - 0.5d0 * grad
           wL(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do i=ix,ix
        wR(i-1,:) = wk(i,:)
     enddo
     
! 2nd order
! van Leer limiter
  elseif( type .eq. 3 ) then

     do i=1,1
        wL(i,:)   = wk(i,:)
     enddo
     do j=1,jx
        do i=2,ix-1
           gA =     ( wk(i,j)  -wk(i-1,j))
           gB =     ( wk(i+1,j)-wk(i,j)  )
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              grad = 2*gA*gB / ( gA + gB )
           endif
           wR(i-1,j) = wk(i,j) - 0.5d0 * grad
           wL(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do i=ix,ix
        wR(i-1,:) = wk(i,:)
     enddo
         
  else
     write(6,*) 'unknown limiter type'
     stop
     
  endif
  
  return
end subroutine limiter_f
