subroutine limiter_g(wk,wD,wU,ix,jx,type)
!-----------------------------------------------------------------------
!     Slope limiters
!-----------------------------------------------------------------------
!     2010/01/22  S. Zenitani  2nd order limiters (minmod, MC)
!     2010/05/14  S. Zenitani  added van Leer limiter
!-----------------------------------------------------------------------
  implicit none
  integer :: ix, jx
  real(8) :: wk(ix,jx)             ! work array (input)
  real(8) :: wD(ix,jx), wU(ix,jx)  ! interpolated values (output)
  integer :: type            ! 0 (flat), 1 (minmod), 2 (MC), 3 (van Leer)
  integer :: i, j
  real(8) :: gA, gB, gC      ! average, left, and right gradients
  real(8) :: grad

! 1st order
  if( type .eq. 0 ) then
     do j=1,jx-1
        wD(:,j) = wk(:,j)
        wU(:,j) = wk(:,j+1)
     enddo

! 2nd order
! minmod limiter
  elseif( type .eq. 1 ) then

     do j=1,1
        wD(:,j)   = wk(:,j)
     enddo
     do j=2,jx-1
        do i=1,ix
           gA =     ( wk(i,j)  -wk(i,j-1))
           gB =     ( wk(i,j+1)-wk(i,j)  )
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              if( gA .gt. 0 ) then
                 grad = min(gA,gB)
              else
                 grad = max(gA,gB)
              endif
           endif
           wU(i,j-1) = wk(i,j) - 0.5d0 * grad
           wD(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do j=jx,jx
        wU(:,j-1) = wk(:,j)
     enddo
         
! 2nd order
! Monotonized central-difference (MC) limitter
  elseif( type .eq. 2 ) then
     
     do j=1,1
        wD(:,j)   = wk(:,j)
     enddo
     do j=2,jx-1
        do i=1,ix
           gA =       ( wk(i,j)  -wk(i,j-1))
           gB =       ( wk(i,j+1)-wk(i,j)  )
           gC = 0.5d0*( wk(i,j+1)-wk(i,j-1))
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              if( gA .gt. 0 ) then
                 grad = min(2*gA,2*gB,gC)
              else
                 grad = max(2*gA,2*gB,gC)
              endif
           endif
           wU(i,j-1) = wk(i,j) - 0.5d0 * grad
           wD(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do j=jx,jx
        wU(:,j-1) = wk(:,j)
     enddo
     
! 2nd order
! van Leer limiter
  elseif( type .eq. 3 ) then

     do j=1,1
        wD(:,j)   = wk(:,j)
     enddo
     do j=2,jx-1
        do i=1,ix
           gA =     ( wk(i,j)  -wk(i,j-1))
           gB =     ( wk(i,j+1)-wk(i,j)  )
           if( gA*gB .le. 0 ) then
              grad = 0.d0
           else
              grad = 2*gA*gB / ( gA + gB )
           endif
           wU(i,j-1) = wk(i,j) - 0.5d0 * grad
           wD(i,j)   = wk(i,j) + 0.5d0 * grad
        enddo
     enddo
     do j=jx,jx
        wU(:,j-1) = wk(:,j)
     enddo

  else
     write(6,*) 'unknown limiter type'
     stop
     
  endif
  
  return
end subroutine limiter_g
